package com.tcctlo.law.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctlo.common.core.domain.model.LoginUser;
import com.tcctlo.common.utils.SecurityUtils;
import com.tcctlo.common.utils.StringUtils;
import com.tcctlo.law.CaseEnum;
import com.tcctlo.law.entity.CaseInformation;
import com.tcctlo.law.entity.CaseLawyer;
import com.tcctlo.law.entity.FileManager;
import com.tcctlo.law.entity.ImpulseInformation;
import com.tcctlo.law.mapper.*;
import com.tcctlo.law.service.ICaseInformationService;
import com.tcctlo.law.service.ICaseLawyerService;
import com.tcctlo.law.service.IFileManagerService;
import com.tcctlo.law.service.IImpulseInformationService;
import com.tcctlo.law.tools.GenerateCaseNo;
import com.tcctlo.law.vo.CaseInformationEnterVO;
import com.tcctlo.law.vo.CaseInformationListVO;
import com.tcctlo.law.vo.CollectionRecordEnterVO;
import com.tcctlo.system.mapper.SysUserMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Consumer;
import java.util.stream.Collectors;

/**
 * <p>
 * 案件信息表 服务实现类
 * </p>
 *
 * @author 张世琪
 * @since 2025-02-06
 */
@Service
public class CaseInformationServiceImpl extends ServiceImpl<CaseInformationMapper, CaseInformation> implements ICaseInformationService {


    @Resource
    private CaseInformationMapper caseInformationMapper;

    @Resource
    private ICaseInformationService caseInformationService;

    /**
     * 系统用户mapper
     */
    @Resource
    private SysUserMapper sysUserMapper;

    /**
     * 利冲信息mapper
     */
    @Resource
    private ImpulseInformationMapper informationMapper;

    /**
     * 利冲信息service
     */
    @Resource
    private IImpulseInformationService informationService;

    /**
     * 案件关联律师mapper
     */
    @Resource
    private CaseLawyerMapper caseLawyerMapper;

    /**
     * 案件关联律师service
     */
    @Resource
    private ICaseLawyerService caseLawyerService;

    /**
     * 文件管理mapper
     */
    @Resource
    private FileManagerMapper fileManagerMapper;

    /**
     * 文件管理service
     */
    @Resource
    private IFileManagerService fileManagerService;

    /**
     * 收款记录mapper
     */
    @Resource
    private CollectionRecordMapper collectionRecordMapper;

    /*@Override
    public Page<CaseInformation> list(Integer pageNo, Integer pageSize, Map<String, Object> params) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String roleKey = sysUserMapper.selectUserRoleByUserId(loginUser.getUserId());

        //如果当前登录用户的角色是【代理律师】或【协办律师】，则查询当前登录律师的案件信息，否则查询全部案件信息
        if ("ATTORNEY_AGENT".equals(roleKey) || "ASSISTING_LAWYER".equals(roleKey)){
            params.put("userId", loginUser.getUserId().toString());
        }
        Page<CaseInformation> page = new Page<>(pageNo, pageSize);
        return caseInformationMapper.selectCondition(page, params);
    }*/


    /*@Override
    public Page<CaseInformationEnterVO> list(Integer pageNo, Integer pageSize, Map<String, Object> params) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String roleKey = sysUserMapper.selectUserRoleByUserId(loginUser.getUserId());

        //如果当前登录用户的角色是【代理律师】或【协办律师】，则查询当前登录律师的案件信息，否则查询全部案件信息
        if ("ATTORNEY_AGENT".equals(roleKey) || "ASSISTING_LAWYER".equals(roleKey)){
            params.put("userId", loginUser.getUserId().toString());
        }
        Page<CaseInformation> page = new Page<>(pageNo, pageSize);
        List<CaseInformation> entityLists = caseInformationMapper.selectCondition(page, params).getRecords();
        Page<CaseInformationEnterVO> resPage = new Page<>();
        List<CaseInformationEnterVO> caseInfoVO = new ArrayList<>();
        for (CaseInformation item : entityLists) {
            CaseInformationEnterVO itemVO = new CaseInformationEnterVO();
            itemVO.setCaseInformation(item);
            itemVO.setEntrustingParty(informationMapper.selectByCaseId(item.getId(), 0));    //利冲信息-委托方信息--list
            itemVO.setOpposite(informationMapper.selectByCaseId(item.getId(), 1));    //利冲信息-相对方--list
            itemVO.setAttorneyAgent(caseLawyerMapper.selectAttorneyAgentLawByCaseId(item.getId()));    //代理律师--entity
            itemVO.setAssistingLawyer(caseLawyerMapper.selectAssistingLawyerByCaseId(item.getId()));    //协办律师--list
            caseInfoVO.add(itemVO);
        }
        resPage.setRecords(caseInfoVO);
        return resPage;
    }*/

    @Override
    public Page<CaseInformationListVO> list(Integer pageNo, Integer pageSize, Map<String, Object> params) {
        if (params == null) {
            params = new HashMap<>();
        }
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String roleKey = sysUserMapper.selectUserRoleByUserId(loginUser.getUserId());

        //如果当前登录用户的角色【律师】，则查询当前登录律师的案件信息，否则查询全部案件信息
        if (CaseEnum.LAWYER.getRoleKey().equals(roleKey)) {
            params.put("userId", loginUser.getUserId().toString());
        }
        Page<CaseInformationListVO> page = new Page<>(pageNo, pageSize);
        Page<CaseInformationListVO> caseInformationPage = caseInformationMapper.selectCondition(page, params);
        for (CaseInformationListVO item : caseInformationPage.getRecords()) {
            item.setEntrustingParty(informationMapper.selectByCaseId(item.getId(), 0).stream().map(ImpulseInformation::getCaseName).collect(Collectors.joining(",")));  //委托方
            item.setOpposite(informationMapper.selectByCaseId(item.getId(), 1).stream().map(ImpulseInformation::getCaseName).collect(Collectors.joining(","))); //相对方
            item.setAttorneyAgent(caseLawyerMapper.selectAttorneyAgentLawByCaseId(item.getId()).getLawyerName());   //代理律师
            item.setAssistingLawyer(caseLawyerMapper.selectAssistingLawyerByCaseId(item.getId()).stream().map(CaseLawyer::getLawyerName).collect(Collectors.joining(","))); //协办律师
        }
        return caseInformationPage;
    }

    @Override
    public CaseInformation getById(Long id) {
        return caseInformationMapper.selectById(id);
    }

    @Override
    public Integer create(CaseInformation caseInformation) {
        return caseInformationMapper.insert(caseInformation);
    }

    @Override
    public Integer delete(Long id) {
        return caseInformationMapper.deleteById(id);
    }

    @Override
    public Integer update(CaseInformation caseInformation) {
        return caseInformationMapper.updateById(caseInformation);
    }

    @Override
    @Transactional(isolation = Isolation.DEFAULT, rollbackFor = Exception.class)
    public Boolean enterCaseInfo(CaseInformationEnterVO caseInfo) {
        try {
            CaseInformation caseInformation = caseInfo.getCaseInformation();   //案件信息
            caseInformation.setCaseNo(new GenerateCaseNo().generateCaseNo()); //生成案件编号
            caseInformationMapper.insert(caseInfo.getCaseInformation());    //插入案件信息
            Long caseId = caseInformation.getId();  //获取案件ID

            //插入案件关联利冲信息【设置委托方和相对方】
            List<ImpulseInformation> impulseInformationList = new ArrayList<>();
            impulseInformationList.addAll(caseInfo.getEntrustingParty().stream().peek(item -> {
                item.setCaseId(caseId);
                item.setIsClient(0);
            }).collect(Collectors.toList()));
            impulseInformationList.addAll(caseInfo.getOpposite().stream().peek(item -> {
                item.setCaseId(caseId);
                item.setIsClient(1);
            }).collect(Collectors.toList()));
            informationService.saveBatch(impulseInformationList);
            List<Long> impulseInformationIds = impulseInformationList.stream().map(item -> item.getId()).collect(Collectors.toList());
            caseInformation.setImpulseInformationIds(StringUtils.join(impulseInformationIds, ","));

            //插入案件关联律师数据
            List<CaseLawyer> caseLawyerList = new ArrayList<>();
            CaseLawyer attorneyAgent = caseInfo.getAttorneyAgent();
            attorneyAgent.setLawyerType(0);
            attorneyAgent.setCaseId(caseId);
            caseLawyerList.add(attorneyAgent); //代理律师

            caseLawyerList.addAll(caseInfo.getAssistingLawyer().stream().peek(item -> {
                item.setCaseId(caseId);
                item.setLawyerType(1);
            }).collect(Collectors.toList()));   //协办律师
            caseLawyerService.saveBatch(caseLawyerList);
            List<Long> caseLawyerIds = caseLawyerList.stream().map(item -> item.getLawyerId()).collect(Collectors.toList());
            caseInformation.setCaseLawyerIds(StringUtils.join(caseLawyerIds, ","));
            caseInformationMapper.updateById(caseInformation); //更新案件表中的律师IDS和利冲信息IDS

            //插入案件关联收款记录信息【收款记录文件插入】
            List<CollectionRecordEnterVO> collectionRecordList = caseInfo.getCollectionRecordList();
            collectionRecordList.forEach(item -> {
                item.setCaseId(caseId);
                List<FileManager> fileManagerList = item.getFileManager().stream().peek(i -> i.setCaseId(caseId)).collect(Collectors.toList());
                fileManagerService.saveBatch(fileManagerList);
                List<Long> fileIds = fileManagerList.stream().map(FileManager::getId).collect(Collectors.toList());
                item.setCollectionUrlId(StringUtils.join(fileIds, ","));
                collectionRecordMapper.insert(item);
            });
        } catch (Exception e) {
            throw new RuntimeException("案件录入失败");
        }
        return true;
    }

    @Override
    public CaseInformationEnterVO selectCaseInfoById(Long id) {
        CaseInformationEnterVO result = new CaseInformationEnterVO();
        //案件信息表----caseInformation
        CaseInformation caseInformation = caseInformationMapper.selectById(id);
        result.setCaseInformation(caseInformation);

        //案件关联【委托方】----entrustingParty
        result.setEntrustingParty(informationMapper.selectByCaseId(id, 0));

        //案件关联【相对方】----opposite
        result.setOpposite(informationMapper.selectByCaseId(id, 1));

        //代理律师----attorneyAgent
        result.setAttorneyAgent(caseLawyerMapper.selectAttorneyAgentLawByCaseId(id));

        //协办律师----assistingLawyer
        result.setAssistingLawyer(caseLawyerMapper.selectAssistingLawyerByCaseId(id));

        /**
         * 收款记录----collectionRecordList
         * 收款记录基本信息
         * 收款记录文件
         */
        List<CollectionRecordEnterVO> records = collectionRecordMapper.selectByCaseId(id);
        result.setCollectionRecordList(records);
        records.stream().peek(item -> {
            List<Long> fileIds = Arrays.stream(item.getCollectionUrlId().split(","))
                    .map(Long::parseLong)
                    .collect(Collectors.toList());
            item.setFileManager(fileManagerMapper.selectBatchIds(fileIds));
        }).collect(Collectors.toList());
        return result;
    }

    @Override
    @Transactional(isolation = Isolation.DEFAULT, rollbackFor = Exception.class)
    public Integer updateCaseInfo(CaseInformationEnterVO params) {
        //案件信息表----caseInformation
        CaseInformation caseInformation = params.getCaseInformation();
        Long caseId = caseInformation.getId();

        //案件关联【委托方】----entrustingParty--从数据库获取出原来的委托方信息与通过接口传入的进行比对
        List<ImpulseInformation> oldImpulse = informationMapper.selectByCaseId(params.getCaseInformation().getId(), 0);
        List<ImpulseInformation> newImpulse = params.getEntrustingParty();
        editImpulseInformation(oldImpulse, newImpulse, caseId);

        //案件关联【相对方】----opposite
        List<ImpulseInformation> oldOpposite = informationMapper.selectByCaseId(params.getCaseInformation().getId(), 1);
        List<ImpulseInformation> newOpposite = params.getOpposite();
        editImpulseInformation(oldOpposite, newOpposite, caseId);
        caseInformation.setImpulseInformationIds(informationMapper.selectByCaseIdAdnDelFlag(params.getCaseInformation().getId()).stream().map(i -> i.getId().toString()).collect(Collectors.joining(",")));


        //代理律师----attorneyAgent----修改原来的律师信息

        CaseLawyer newAttorneyAgent = params.getAttorneyAgent();
        if (newAttorneyAgent.getId() == null) {
            CaseLawyer oldCaseLawyer = caseLawyerMapper.selectAttorneyAgentLawByCaseId(params.getCaseInformation().getId());
            oldCaseLawyer.setIsPrime(1);
            caseLawyerMapper.updateById(oldCaseLawyer);
            newAttorneyAgent.setCaseId(params.getCaseInformation().getId());
            caseLawyerMapper.insert(newAttorneyAgent);
        }


        //协办律师----assistingLawyer
        List<CaseLawyer> newAssistingLawyer = params.getAssistingLawyer(); //协办律师
        List<CaseLawyer> oldAssistingLawyer = caseLawyerMapper.selectAssistingLawyerByCaseId(params.getCaseInformation().getId());
        editAssistingLawyer(oldAssistingLawyer, newAssistingLawyer, caseId);
        caseInformation.setCaseLawyerIds(caseLawyerMapper.selectAllPrimeLawyerByCaseId(params.getCaseInformation().getId()).stream().map(c -> c.getLawyerId().toString()).collect(Collectors.joining(",")));
        caseInformationMapper.updateById(caseInformation);
        /**
         * 收款记录----collectionRecordList
         * 收款记录基本信息----collectionRecord
         * 收款记录文件----fileManager
         */
        return 0;
    }

    @Override
    public Boolean deleteCase(String ids) {
        List<Long> longList = Arrays.stream(ids.split(","))
                .map(Long::parseLong)
                .collect(Collectors.toList());
        return caseInformationService.removeBatchByIds(longList);
    }

    /**
     * 编辑协办律师
     *
     * @param oldData 旧的数据
     * @param newData 新的数据
     */
    public void editAssistingLawyer(List<CaseLawyer> oldData, List<CaseLawyer> newData, Long caseId) {
        List<CaseLawyer> insert = newData.stream().filter(c -> c.getId() == null).peek(c -> c.setCaseId(caseId)).collect(Collectors.toList());
        List<Long> oldIds = oldData.stream().map(c -> c.getId()).collect(Collectors.toList());
        List<Long> newIds = newData.stream().map(c -> c.getId()).collect(Collectors.toList());
        List<Long> deleteIds = (List<Long>) CollUtil.subtract(oldIds, newIds); //将原来的数据与现在数据进行比较，得到需要删除的ID
        if (CollUtil.isNotEmpty(insert)) {
            caseLawyerService.saveBatch(insert);
        }
        if (CollUtil.isNotEmpty(deleteIds)) {
            caseLawyerMapper.updateByIds(deleteIds);
        }
    }

    /**
     * 编辑利冲信息
     *
     * @param oldData 旧的数据
     * @param newData 新的数据
     */
    public void editImpulseInformation(List<ImpulseInformation> oldData, List<ImpulseInformation> newData, Long caseId) {
        List<ImpulseInformation> insert = newData.stream().filter(item -> item.getId() == null).peek(item -> item.setCaseId(caseId)).collect(Collectors.toList());
        List<Long> oldIds = oldData.stream().map(item -> item.getId()).collect(Collectors.toList());
        List<Long> newIds = newData.stream().map(item -> item.getId()).collect(Collectors.toList());
        Collection<Long> updateIds = CollUtil.intersection(oldIds, newIds); //交集【UPDATE】
        Collection<Long> deleteIds = CollUtil.subtract(oldIds, newIds); //差集【DELETE】

        List<ImpulseInformation> updateDate = newData.stream().filter(item -> CollUtil.contains(updateIds, item.getId())).collect(Collectors.toList());
        List<ImpulseInformation> deleteData = oldData.stream().filter(item -> CollUtil.contains(deleteIds, item.getId())).collect(Collectors.toList());

        /*if (!StringUtils.isEmpty(insert)){
            informationService.saveBatch(insert);
        }
        if (!StringUtils.isEmpty(updateDate)){
            informationService.updateBatchById(updateDate);
        }
        if (!StringUtils.isEmpty(deleteData)){
            informationMapper.deleteBatchLogic(deleteData);
        }*/
        if (CollUtil.isNotEmpty(insert)) {
            informationService.saveBatch(insert);
        }
        if (CollUtil.isNotEmpty(updateDate)) {
            informationService.updateBatchById(updateDate);
        }
        if (CollUtil.isNotEmpty(deleteData)) {
            informationMapper.deleteBatchLogic(deleteData);
        }
    }

}
